#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Particle Systems" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from wx import *
from wx.glcanvas import *

from part_engine import *


TIMER_MS = 25


class myGLCanvas(GLCanvas):
    def __init__(self, parent):
        """
        Création du canevas recevant la modélisation 3D
        """
        GLCanvas.__init__(self, parent,-1)

        self.init = False
        self.context = GLContext(self)

        glutInit()
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        
        self.Bind(EVT_PAINT, self.OnPaint)
        self.Bind(EVT_SIZE, self.OnResize)
        self.Bind(EVT_KEY_DOWN, self.handleKeypress)
         

    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)


    def OnPaint(self, event):
        if not self.init :
            self.init = True

            self.SetCurrent(self.context)
            
            self.textureId = glGenTextures(1)         
            self.particuleEngine = ParticuleEngine(self.textureId)
            self.initRendering()
            
            self.loadTexture()       
            self.update(0)
            self.OnResize(0)
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        glTranslatef(0.0, 0.0, -10.0)
        glScalef(2.0, 2.0, 2.0)
        
        self.particuleEngine.draw()

        self.SwapBuffers()
           

    def OnResize(self, event):
        if not self.init: return

        w, h = self.GetSize()

        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)

        self.Refresh()


    def handleKeypress(self, event):
        key = event.GetUnicodeKey()
        if key == 27 : exit()


    def loadTexture(self):
        try:
            img = Image("circle.bmp", wx.BITMAP_TYPE_ANY) 
            imgAlpha = Image("circlealpha.bmp", wx.BITMAP_TYPE_ANY)
        
            w, h = img.GetSize()
        except:
            return
        
        pixels = self.addAlphaChannel(img, imgAlpha)    
        
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexImage2D(GL_TEXTURE_2D, \
                     0, \
                     GL_RGBA, \
                     w, h, \
                     0, \
                     GL_RGBA, \
                     GL_UNSIGNED_BYTE, \
                     pixels)

        del img
        del imgAlpha
        del pixels


    def addAlphaChannel(self, img, imgAlpha):
        w, h = img.GetSize()

        pixels = []
        for y in range (h) :
            for x in range (w) :
                color = [img.GetRed(x,y), \
                         img.GetGreen(x, y), \
                         img.GetBlue(x, y)]
                color.append(imgAlpha.GetBlue(x, y))
                pixels.append(color)

        return pixels


    def update(self, value):
        self.particuleEngine.advance(TIMER_MS/1000.)

        self.OnPaint(1)
        
        CallLater(25, self.update, 0)

        
class test(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, \
                          title = "Particle Systems - videotutorialsrock.com", \
                          size = (506, 432))
        aire = myGLCanvas(self)
        panel = wx.Panel(self, -1, size = (100, 432))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(aire, 1, wx.EXPAND)
        sizer.Add(panel, 0)
        
        self.SetSizer(sizer)

        self.Show()


if __name__ == '__main__' :
    app = wx.App()
    frame = test()
    frame.Show()
    app.MainLoop()
